//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  Controller.hpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "EpsonScan2.h"
#include "Engine.hpp"
#include "TransferEvent.hpp"
#include "FFManager.hpp"
#include <memory>
#include <vector>
#include <string>
namespace epsonscan {
class Scanner;
class ModelInfo;
class TransferMgr;
class KeyMgr;
class Image;
class Controller {
  public:
    Controller(const SDIDeviceInfo& devInfo, SDIScannerDriver* driver);
    virtual ~Controller();
    SDIError Open();
    void Close();
    void Reset();
    void SetValue(const std::string& key, SDIValueType valType, void* value, SDIInt size);
    void GetValue(const std::string& key, SDIValueType valType, void* value, SDIInt size);
    void GetDefault(const std::string& key, SDIValueType valType, void* value, SDIInt size);
    void GetCapability(const std::string& key, SDICapability& capability);
    SDIError Scan();
    void Cancel();
    boost::optional<TransferEvent> DequeueEvent() ;
    bool CheckNextEvent(bool waitUntilReceived);
    void SetInterruptEventCallBackProc(InterruptEventCallBackProc interruptEventCallBack, void* interruptEventCallBackUserData)
    {
     interruptEventCallBack_ = interruptEventCallBack;
     interruptEventCallBackUserData_ = interruptEventCallBackUserData;
    }
    void SetInterruptionEnabled(bool interruptionEnabled)
    {
     interruptionEnabled_ = interruptionEnabled;
    }
    void CheckGlassDirtStatus();
    bool SDIMultipage_Start(const char* destPath, SDIImageFormat format)
    {
     return FFManager::GetInstance().MultipageStart(destPath, format, keyMgr_);
    }
    bool SDIMultipage_Add(const char* sourcePath)
    {
     return FFManager::GetInstance().MultipageAdd(sourcePath);
    }
    bool SDIMultipage_Fin()
    {
     return FFManager::GetInstance().MultipageFin();
    }
  private:
    void ScannerCallBackReceiver(EngineEventType type, Image* image, SDIError error);
    std::shared_ptr<ModelInfo> modelInfo_;
    std::shared_ptr<Scanner> scanner_;
    std::shared_ptr<TransferMgr> transferMgr_;
    std::shared_ptr<KeyMgr> keyMgr_;
    bool opened_;
    bool interruptionEnabled_;
    SDIScannerDriver* driver_;
    InterruptEventCallBackProc interruptEventCallBack_;
    void* interruptEventCallBackUserData_;
};
}
