//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  DTR.hpp -- template and derived ESC/I ESC/I-2 protocol commands
#pragma once

       
#include <utils/ESBuffer.h>
#include <ES2Command/ES2CommandAPI.h>
#include "ESCommonTypedef.h"
#include "ESImageInfo.h"
#include "EpsonScan2.h"
#include "ModelInfo.hpp"
#include "Filter.hpp"
#include "Utility.hpp"
namespace epsonscan
{
enum ActionType : ESNumber
{
    kActionTypeGetDocumentLocation,
    kActionTypeJudgeBlankPage
};
class DTR : public Filter
{
public:
    DTR(std::shared_ptr<KeyMgr> keyMgr,
          std::shared_ptr<ModelInfo> modelInfo) : Filter(keyMgr, modelInfo),
                                                  actionType_(kActionTypeGetDocumentLocation),
                                                  isBlankPage_(false)
    {
    }
    void SetActionType(ActionType type)
    {
        actionType_ = type;
    }
    virtual void DoProcess(ESImageInfo& imageInfo, ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf) override
    {
        if (actionType_ == kActionTypeGetDocumentLocation )
        {
            SDI_TRACE_LOG("keyMgr_->GetCommandValueAsBool(kESAutoCropping) = %d", keyMgr_->GetCommandValueAsBool(kESAutoCropping));
            if ( keyMgr_->GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitDocumentFeeder &&
                (keyMgr_->GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeLongPaper ||
                (keyMgr_->GetCommandValueAsBool(kESAutoCropping) && ES_IMAGE_INFO::GetESImageAutoCropped(imageInfo) == FALSE)))
            {
                    DoAutoCrop(imageInfo,
                               inDataBuf,
                               modelInfo_,
                               keyMgr_->GetWorkTempPath());
            }
        }
        if (actionType_ == kActionTypeJudgeBlankPage )
        {
   if (keyMgr_->GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitDocumentFeeder &&
                (keyMgr_->GetValueInt(kSDIBlankPageSkipKey) != 0 &&
                keyMgr_->GetValueInt(kSDIBlankPageSkipLevelKey) != 0))
   {
      if(JudgeBlankPage(imageInfo,
              inDataBuf,
            modelInfo_,
            keyMgr_->GetValueInt(kSDIBlankPageSkipLevelKey),
            keyMgr_->GetWorkTempPath()) == true)
     {
       SDI_TRACE_LOG("page = %d is blankpage", ES_IMAGE_INFO::GetESImagePageCount(imageInfo));
       isBlankPage_ = true;
     }
   }
        }
    }
    bool IsBlankPage()
    {
        return isBlankPage_;
    }
    std::string Name() override
    {
        return "DTR";
    }
    static bool IsDTRAvailable();
private:
 static SDIError DoAutoCrop(ESImageInfo& imageInfo,
                   ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf,
                               std::shared_ptr<ModelInfo> modelInfo,
                               std::string workFolder
                               );
    static bool JudgeBlankPage(ESImageInfo& imageInfo,
                   ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf,
                               std::shared_ptr<ModelInfo> modelInfo,
                               ESNumber skipLevel,
                               std::string workFolder);
    ActionType actionType_;
    bool isBlankPage_;
};
}
