//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  AutoSize.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "AutoSize.hpp"
#include "Scanner.hpp"
#include "ModelInfo.hpp"
#include "DTR.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void AutoSize::SetValue(const SDIInt& intVal)
{
 SDI_TRACE_LOG("SetValue = %d", intVal);;
 current_ = intVal;
}
void AutoSize::Reset()
{
 SDICapability cap = {0};
 GetCapability(cap);
 if (cap.supportLevel != kSDISupportLevelNone)
 {
  current_ = kSDIAutoSizeStandard;
 }
 else
 {
  current_ = kSDIAutoSizeNone;
 }
}
void AutoSize::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
bool AutoSize::IsHardwareAvailable()
{
 SDI_TRACE_LOG("Enter");
 Scanner* scanner = dataProvider_->GetScanner().get();
 if (GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitDocumentFeeder)
 {
   ESIndexSet availaleValues;
   scanner->GetAvailableValueForKey(kESAutoCropping, availaleValues);
   if (availaleValues.find(true) != availaleValues.end()) {
    SDI_TRACE_LOG("Enter true");
    return true;
   }
   else
   {
    SDI_TRACE_LOG("Leave false");
    return false;
   }
 }
 SDI_TRACE_LOG("Leave false");
  return false;
}
bool AutoSize::IsSoftwareAvailable()
{
 SDI_TRACE_LOG("Enter");
 if (GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitDocumentFeeder)
 {
  ModelInfo* modelInfo = dataProvider_->GetModelInfo().get();
  if (modelInfo)
  {
   ESNumber autoSize;
   modelInfo->GetValue(kPrvHRD_SWAutoSize, autoSize);
   SDI_TRACE_LOG("Get kPrvHRD_SWAutoSize  = %d", autoSize);
   return autoSize && DTR::IsDTRAvailable();
  }
 }
 SDI_TRACE_LOG("Leave false");
 return false;
}
bool AutoSize::IsLongPaperAvailable()
{
 if (GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitDocumentFeeder)
 {
  ModelInfo* modelInfo = dataProvider_->GetModelInfo().get();
  if (modelInfo)
  {
   ESNumber autoSize;
   modelInfo->GetValue(kPrvHRD_AutoDetectLongPaper, autoSize);
   return autoSize;
  }
 }
 return false;
}
void AutoSize::GetCapability(SDICapability &capability)
{
 SetDefaultListCapability(capability);
 capability.supportLevel = kSDISupportLevelNone;
 if (IsSoftwareAvailable() || IsHardwareAvailable())
 {
  AddList(capability, kSDIAutoSizeStandard);
  AddList(capability, kSDIAutoSizeNone);
  AddAllList(capability, kSDIAutoSizeStandard);
  AddAllList(capability, kSDIAutoSizeNone);
  if (IsLongPaperAvailable() && IsSoftwareAvailable() )
  {
   AddList(capability, kSDIAutoSizeLongPaper);
   AddAllList(capability, kSDIAutoSizeLongPaper);
  }
  capability.supportLevel = kSDISupportLevelAvailable;
 }else{
  AddList(capability, kSDIAutoSizeNone);
  AddAllList(capability, kSDIAutoSizeNone);
 }
}
ESDictionary AutoSize::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 if (current_ == kSDIAutoSizeStandard)
 {
  dict[kESAutoCropping] = (bool)true;
 }else{
  dict[kESAutoCropping] = (bool)false;
 }
 return dict;
}
}
