//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  ColorType.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "ColorType.hpp"
#include "Scanner.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void ColorType::SetValue(const SDIInt& intVal)
{
 current_ = intVal;
}
void ColorType::Reset()
{
 current_ = kSDIColorTypeMono1;
}
void ColorType::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
void ColorType::GetCapability(SDICapability &capability)
{
 SetDefaultListCapability(capability);
 capability.supportLevel = kSDISupportLevelAvailable;
 ESIndexSet availableColorTypes;
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESColorFormat, availableColorTypes)) {
  AddAllList(capability, kSDIColorTypeRGB24);
  AddList(capability, kSDIColorTypeRGB24);
  AddAllList(capability, kSDIColorTypeMono8);
  AddList(capability, kSDIColorTypeMono8);
  AddAllList(capability, kSDIColorTypeMono1);
  AddList(capability, kSDIColorTypeMono1);
  if (availableColorTypes.find(kESColorFormatRGB48) != availableColorTypes.end())
     {
   AddList(capability, kSDIColorTypeRGB48);
   AddAllList(capability, kSDIColorTypeRGB48);
  }
  if (availableColorTypes.find(kESColorFormatMono16) != availableColorTypes.end())
     {
   AddList(capability, kSDIColorTypeMono16);
   AddAllList(capability, kSDIColorTypeMono16);
  }
 }
}
bool ColorType::IsHardawaDropout()
{
 SDIDropoutColor dropout = (SDIDropoutColor)GetValueInt(kSDIDropoutColorKey);
 if (dropout == kSDIDropoutColorNone)
 {
  return false;
 }
 if (GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeLongPaper)
 {
  return false;
 }
 return true;
}
bool ColorType::IsSoftwareDropout()
{
 return IsHardawaDropout() == false;
}
ESNumber ColorType::GetColorType()
{
 Scanner* scanner = dataProvider_->GetScanner().get();
 ESIndexSet availableColorTypes;
 ESColorFormat colorFormat = kESColorFormatRGB24;
 if (current_ == kSDIColorTypeMono1)
 {
  colorFormat = kESColorFormatMono8;
 }
 if (current_ == kSDIColorTypeMono8)
 {
  colorFormat = kESColorFormatMono8;
 }
 if (current_ == kSDIColorTypeRGB24)
 {
  colorFormat = kESColorFormatRGB24;
 }
 if (current_ == kSDIColorTypeRGB48)
 {
  colorFormat = kESColorFormatRGB48;
 }
 SDIDropoutColor dropout = (SDIDropoutColor)GetValueInt(kSDIDropoutColorKey);
 if (IsHardawaDropout())
 {
  if (dropout == kSDIDropoutColorRed)
  {
   if (colorFormat == kESColorFormatMono8)
   {
    colorFormat = kESColorFormatMonoDropR8;
   }
   if (colorFormat == kESColorFormatMono1)
   {
    colorFormat = kESColorFormatMonoDropR1;
   }
  }
  if (dropout == kSDIDropoutColorGreen)
  {
   if (colorFormat == kESColorFormatMono8)
   {
    colorFormat = kESColorFormatMonoDropG8;
   }
   if (colorFormat == kESColorFormatMono1)
   {
    colorFormat = kESColorFormatMonoDropG1;
   }
  }
  if (dropout == kSDIDropoutColorBlue)
  {
   if (colorFormat == kESColorFormatMono8)
   {
    colorFormat = kESColorFormatMonoDropB8;
   }
   if (colorFormat == kESColorFormatMono1)
   {
    colorFormat = kESColorFormatMonoDropB1;
   }
  }
 }
 else
 {
  colorFormat = kESColorFormatRGB24;
 }
 return colorFormat;
}
void ColorType::GetGammaTable(ESGammaTable& outGammaTable)
{
 ESIndexArray narRGamma;
 ESIndexArray narGGamma;
 ESIndexArray narBGamma;
    for(ESNumber i = 0; i < kESGammaTableMax256; i++)
 {
  narRGamma.push_back(i);
  narGGamma.push_back(i);
  narBGamma.push_back(i);
    }
    outGammaTable[kESGammaTableMaxKey] = (ESNumber)kESGammaTableMax256;
    outGammaTable[kESGammaTableRedChannelKey] = narRGamma;
    outGammaTable[kESGammaTableGreenChannelKey] = narGGamma;
    outGammaTable[kESGammaTableBlueChannelKey] = narBGamma;
    outGammaTable[kESGammaTableMonoChannelKey] = narGGamma;
}
ESDictionary ColorType::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 ESColorFormat colorFormat = static_cast<ESColorFormat>(GetColorType());
 if ((colorFormat & kESColorFormatMaskBitDepth) == 8)
 {
  dict[kESGammaMode] = (ESNumber)kESGammaMode18;
 }else{
  dict[kESGammaMode] = (ESNumber)kESGammaMode10;
 }
 dict[kESColorFormat] = (ESNumber)colorFormat;
 ESGammaTable gammaTable;
 GetGammaTable(gammaTable);
 if (colorFormat == kESColorFormatRGB24)
 {
  dict[kESGammaTableRed] = gammaTable[kESGammaTableRedChannelKey];
  dict[kESGammaTableGreen] = gammaTable[kESGammaTableGreenChannelKey];
  dict[kESGammaTableBlue] = gammaTable[kESGammaTableBlueChannelKey];
  dict[kESColorMatrix] = (ESNumber)kESColorMatrixUnit;
 }else{
  dict[kESGammaTableMono] = gammaTable[kESGammaTableMonoChannelKey];
 }
 return dict;
}
}
