//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  Dropout.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "Dropout.hpp"
#include "Scanner.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void Dropout::SetValue(const SDIInt& intVal)
{
 current_ = intVal;
}
void Dropout::Reset()
{
 current_ = kSDIDropoutColorNone;
}
void Dropout::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
void Dropout::GetCapability(SDICapability &capability)
{
 SetDefaultListCapability(capability);
 ESIndexSet availableColorTypes;
   capability.supportLevel = kSDISupportLevelNone;
 if (GetValueInt(kSDIColorTypeKey) == kSDIColorTypeRGB24 ||
  GetValueInt(kSDIColorTypeKey) == kSDIColorTypeRGB48)
 {
  capability.supportLevel = kSDISupportLevelUnavailable;
  AddAllList(capability, kSDIDropoutColorRed);
  AddAllList(capability, kSDIDropoutColorGreen);
  AddAllList(capability, kSDIDropoutColorBlue);
  return;
 }
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESColorFormat, availableColorTypes)) {
  AddList(capability, kSDIDropoutColorNone);
  AddAllList(capability, kSDIDropoutColorNone);
  if (availableColorTypes.find(kESColorFormatMonoDropR8) != availableColorTypes.end())
     {
   AddList(capability, kSDIDropoutColorRed);
   AddAllList(capability, kSDIDropoutColorRed);
   capability.supportLevel = kSDISupportLevelAvailable;
  }
  if (availableColorTypes.find(kESColorFormatMonoDropG8) != availableColorTypes.end())
     {
   AddList(capability, kSDIDropoutColorGreen);
   AddAllList(capability, kSDIDropoutColorGreen);
   capability.supportLevel = kSDISupportLevelAvailable;
  }
  if (availableColorTypes.find(kESColorFormatMonoDropB8) != availableColorTypes.end())
     {
   AddList(capability, kSDIDropoutColorBlue);
   AddAllList(capability, kSDIDropoutColorBlue);
   capability.supportLevel = kSDISupportLevelAvailable;
  }
 }
}
ESDictionary Dropout::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 return dict;
}
}
