//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  FunctionalUnit.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "FunctionalUnit.hpp"
#include "Scanner.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void FunctionalUnit::SetValue(const SDIInt& intVal)
{
 current_ = intVal;
}
void FunctionalUnit::Reset()
{
 if (ADFAvailable())
 {
  current_ = kSDIFunctionalUnitDocumentFeeder;
  return;
 }
 if (FlatbedAvailable())
 {
  current_ = kSDIFunctionalUnitFlatbed;
  return;
 }
}
void FunctionalUnit::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
bool FunctionalUnit::ADFAvailable()
{
 ESIndexSet availableFunctionalUnit;
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESFunctionalUnitType, availableFunctionalUnit)) {
  if (availableFunctionalUnit.find(kESFunctionalUnitDocumentFeeder) != availableFunctionalUnit.end()) {
   return true;
  }
 }
 return false;
}
bool FunctionalUnit::FlatbedAvailable()
{
 ESIndexSet availableFunctionalUnit;
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESFunctionalUnitType, availableFunctionalUnit)) {
  if (availableFunctionalUnit.find(kESFunctionalUnitFlatbed) != availableFunctionalUnit.end()) {
   return true;
  }
 }
 return false;
}
void FunctionalUnit::GetCapability(SDICapability &capability)
{
 SetDefaultListCapability(capability);
    capability.supportLevel = kSDISupportLevelAvailable;
 if (ADFAvailable()) {
  AddList(capability, kSDIFunctionalUnitDocumentFeeder);
  AddAllList(capability, kSDIFunctionalUnitDocumentFeeder);
 }
 if (FlatbedAvailable()) {
  AddList(capability, kSDIFunctionalUnitFlatbed);
  AddAllList(capability, kSDIFunctionalUnitFlatbed);
 }
}
ESDictionary FunctionalUnit::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 Scanner* scanner = dataProvider_->GetScanner().get();
 if (current_ == kSDIFunctionalUnitDocumentFeeder)
 {
  dict[kESFunctionalUnitType] = (ESNumber)kESFunctionalUnitDocumentFeeder;
  scanner->SetValueForKey(kESFunctionalUnitType, (ESNumber)kESFunctionalUnitDocumentFeeder);
 }
 if (current_ == kSDIFunctionalUnitFlatbed)
 {
  dict[kESFunctionalUnitType] = (ESNumber)kESFunctionalUnitFlatbed;
  scanner->SetValueForKey(kESFunctionalUnitType, (ESNumber)kESFunctionalUnitFlatbed);
 }
 return dict;
}
}
