//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  event_caller.hpp -- template and derived ESC/I ESC/I-2 protocol commands
#pragma once

       
#include <condition_variable>
#include <mutex>
#include <future>
#include <thread>
#include <chrono>
#include <functional>
class event_caller
{
public:
  event_caller(std::function<void()> call, int interval_msec = 1)
  : call_(call)
  , interval_msec_(interval_msec)
  , is_exit_(false)
  , is_stop_(false)
  {
  }
  virtual ~event_caller()
  {
    is_exit_ = true;
    try{
      cond_.notify_one();
    }catch(...){
    }
  }
  bool stop(){return is_stop_;}
  void stop(bool stop){ is_stop_ = stop;}
  void start()
  {
    task_ = std::async(std::launch::async, [this] { event_loop_(); });
  }
protected:
  void event_loop_(){
    std::unique_lock<std::mutex> lk(mtx_);
    while (call_ && !is_exit_) {
      try{
        using namespace std::chrono;
        steady_clock::time_point tp = steady_clock::now() + milliseconds(interval_msec_);
        std::cv_status result = cond_.wait_until(lk, tp);
        if (!is_stop_ && !is_exit_){
          call_();
        }
      } catch(...){
        break;
      }
    }
  }
private:
  std::function<void()> call_;
  int interval_msec_;
  std::future<void> task_;
  std::mutex mtx_;
  std::condition_variable cond_;
  std::atomic_bool is_exit_;
  std::atomic_bool is_stop_;
};
