//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  message.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "message.h"
#if ENABLE_DEBUG
unsigned long msg_level = 0;
#include <ctype.h>
#include <stdlib.h>
#include <strings.h>
void
msg_init (void)
{
  struct level_def
  {
    const char *key;
    msg_level_type val;
  };
  const struct level_def def[] =
    {
      {"FATAL", ERR_FATAL},
      {"MAJOR", ERR_MAJOR},
      {"MINOR", ERR_MINOR},
      {"INFO" , LOG_INFO},
      {"CALL" , LOG_CALL},
      {"DATA" , LOG_DATA},
      {"CMD" , DBG_CMD},
      {"HEX" , DBG_HEX},
      {"IMG" , DBG_IMG},
      {NULL}
    };
  const char *level = getenv ("SANE_DEBUG_EPSONSCAN");
  const struct level_def *p = def;
  msg_level = 0;
  if (!level) return;
  while (p && p->key)
    {
      if (0 == strcasecmp (level, p->key))
        {
          msg_level = p->val;
          log_info ("setting message level to '%s' (%d)",
                    p->key, p->val);
          return;
        }
      ++p;
    }
}
void
msg_dump (const char *fmt, const void *buffer, size_t sz)
{
  const size_t quad_length = 4;
  const size_t quad_count = 4;
  const size_t line_length = quad_length * quad_count;
  const unsigned char *buf = (const unsigned char *)buffer;
  char ascii[line_length + 1];
  size_t i = 0;
  ascii[line_length] = '\0';
  while (i < sz)
    {
      if (0 == i % line_length)
        fprintf (stderr, "%s%08zx: ", fmt, i);
      ascii[i % line_length] = (isprint (buf[i]) ? buf[i] : '.');
      fprintf (stderr, " %02x", buf[i]);
      ++i;
      if (0 == i % quad_length)
        fprintf (stderr, " ");
      if (0 == i % line_length)
        fprintf (stderr, " |%s|\n", ascii);
    }
  if (0 != i % line_length)
    {
      do
        {
          ascii[i % line_length] = ' ';
          fprintf (stderr, "   ");
          ++i;
          if (0 == i % quad_length)
            fprintf (stderr, " ");
        }
      while (0 != i % line_length);
      fprintf (stderr, " |%s|\n", ascii);
    }
}
#endif
