//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  configurationdialog.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "configurationdialog.h"
#include "configuration.h"
#include "ui_configurationdialog.h"
#include <QMessageBox>
#include "lastusedsettings.h"
#include "mainwindow.h"
#include "PathUtils.h"
#include "../Include/Config.h"
#define NEAREND 2000
ConfigurationDialog::ConfigurationDialog(QWidget *parent) :
    QDialog(parent),
    config_ui(new Ui::ConfigurationDialog)
{
    config_ui->setupUi(this);
    this->setFixedSize(this->size());
    this->updateUI();
    this->bReset = false;
}
ConfigurationDialog::~ConfigurationDialog()
{
    delete config_ui;
}
void ConfigurationDialog::onValueChanged(int value)
{
    QObject* obj = sender();
    if(obj == config_ui->Units_comboBox){
        config_ui->Units_comboBox->item_event(value);
    }else if(obj == config_ui->Sleep_Timer_spinBox){
        config_ui->Sleep_Timer_spinBox->item_event(value);
    }else if(obj == config_ui->AC_Power_comboBox){
        config_ui->AC_Power_comboBox->item_event(value);
    }else if(obj == config_ui->Battery_Power_comboBox){
        config_ui->Battery_Power_comboBox->item_event(value);
    }
    this->updateUI();
}
void ConfigurationDialog::onValueChangedBool(bool value)
{
    QObject* obj = sender();
    if(obj == config_ui->Last_Used_checkBox){
        config_ui->Last_Used_checkBox->item_event(value);
    }else if(obj == config_ui->Transfer_data_after_comp_checkBox){
        config_ui->Transfer_data_after_comp_checkBox->item_event(value);
    }else if(obj == config_ui->Open_Img_Folder_after_scan_checkBox){
        config_ui->Open_Img_Folder_after_scan_checkBox->item_event(value);
    }
    this->updateUI();
}
void ConfigurationDialog::Initialize()
{
    Configuration& config = Configuration::GetConfiguration();
    config_ui->Units_comboBox->initialize(config.config_para.units);
    config_ui->Last_Used_checkBox->initialize(config.config_para.last_used_settings);
    config_ui->Transfer_data_after_comp_checkBox->initialize(config.config_para.compression);
    config_ui->Open_Img_Folder_after_scan_checkBox->initialize(config.config_para.open_img_folder);
    config_ui->AC_Power_comboBox->initialize(config.config_para.ac_power);
    config_ui->Battery_Power_comboBox->initialize(config.config_para.usb_power);
    config_ui->Sleep_Timer_spinBox->initialize(config.config_para.sleep_timer);
    this->updateUI();
}
void ConfigurationDialog::updateUI(){
    config_ui->Units_comboBox->update_ui();
    config_ui->Last_Used_checkBox->update_ui();
    config_ui->Transfer_data_after_comp_checkBox->update_ui();
    config_ui->Open_Img_Folder_after_scan_checkBox->update_ui();
    config_ui->AC_Power_comboBox->update_ui();
    config_ui->Battery_Power_comboBox->update_ui();
}
void ConfigurationDialog::UpdateCounterDisplay(CONFIGURE_COUNTER_VALUE ConfigureCounter)
{
    CONFIGURATION config_val;
    Configuration& config = Configuration::GetConfiguration();
    config_val = config.config_para;
    int count_num = ConfigureCounter.ScanCounterKey.select;
    if(count_num != 0){
        config_ui->Scanner_Glass_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Scanner_Glass_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.ADFScanCounterKey.select + ConfigureCounter.ADFDuplexScanCounterKey.select;
    if(count_num != 0){
        config_ui->ADF_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->ADF_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.ADFScanCounterKey.select;
    if(count_num != 0){
        config_ui->Single_Sided_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Single_Sided_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.ADFDuplexScanCounterKey.select;
    if(count_num != 0){
        config_ui->Double_Sided_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Double_Sided_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.ADFCarrierSheetScanCounterKey.select;
    if(count_num != 0){
        config_ui->Carrier_Sheet_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Carrier_Sheet_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.CardScanCounterKey.select;
    if(count_num != 0){
        config_ui->Card_Slot_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Card_Slot_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.PaperJamCounter.select;
    if(count_num != 0){
        config_ui->Paper_Jam_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Paper_Jam_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.DoubleFeedCounter.select;
    if(count_num != 0){
        config_ui->Double_Feed_cnt_label->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Double_Feed_cnt_label->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    count_num = ConfigureCounter.RollerCounter.select;
    config_val.roller_counter = count_num;
    config_ui->Number_Of_Scans_cnt_label3->setText(std::to_string(count_num).c_str());
    count_num = ConfigureCounter.DurationCounter.select;
    if(count_num != 0){
        config_ui->Life_Cyde_cnt_label3->setText(std::to_string(count_num).c_str());
    }else {
        config_ui->Life_Cyde_cnt_label3->setText(TranslationString::GetString().translation_str.ES2U_MAIN_008);
    }
    config.SetConfiguration(config_val);
}
void ConfigurationDialog::PushButtonAction(void)
{
    QObject* obj = sender();
    if(obj == config_ui->Other_Reset_pushButton){
        if(this->ResetMessageBox()){
            std::string settingDir = qPrintable(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
            ES_CMN_FUNCS::PATH::ES_CombinePath(settingDir, settingDir, ".epsonscan2/Settings/");
            ES_CMN_FUNCS::PATH::ES_DeleteFolder(settingDir.c_str());
            ES_CMN_FUNCS::PATH::ES_MakeFolder(settingDir.c_str());
            this->bReset = true;
            this->reject();
        }
    }else if(obj == config_ui->Counter_Reset_pushButton){
        if(this->CounterResetMessageBox()){
            CONFIGURATION config_val;
            Configuration& config = Configuration::GetConfiguration();
            config_val = config.config_para;
            config_ui->Number_Of_Scans_cnt_label3->setText("0");
            config_val.roller_counter = 0;
            config.SetConfiguration(config_val);
        }
    }
}
bool ConfigurationDialog::Get_ResetFlag()
{
    return this->bReset;
}
bool ConfigurationDialog::ResetMessageBox(void)
{
    QMessageBox msgBox(this);
    msgBox.setText(TranslationString::GetString().translation_str.LINUX_C_UI_001);
    msgBox.setWindowTitle(TranslationString::GetString().translation_str.MENU_001);
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    msgBox.setButtonText(QMessageBox::Yes, TranslationString::GetString().translation_str.ES2U_MAIN_052);
    msgBox.setButtonText(QMessageBox::No, TranslationString::GetString().translation_str.ES2U_MAIN_053);
    msgBox.setIcon(QMessageBox::Warning);
    int res = msgBox.exec();
    if (res == QMessageBox::Yes){
        return true;
    } else if (res == QMessageBox::No) {
        return false;
    }
    return false;
}
bool ConfigurationDialog::CounterResetMessageBox(void)
{
    QMessageBox msgBox(this);
    msgBox.setText(TranslationString::GetString().translation_str.C_UNIT_001);
    msgBox.setWindowTitle(TranslationString::GetString().translation_str.MENU_001);
    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    msgBox.setButtonText(QMessageBox::Yes, TranslationString::GetString().translation_str.ES2U_MAIN_052);
    msgBox.setButtonText(QMessageBox::No, TranslationString::GetString().translation_str.ES2U_MAIN_053);
    msgBox.setIcon(QMessageBox::Warning);
    int res = msgBox.exec();
    if (res == QMessageBox::Yes){
        return true;
    } else if (res == QMessageBox::No) {
        return false;
    }
    return false;
}
void ConfigurationDialog::DisplayItem(Supervisor* sv)
{
    if(sv->device_data.TransferCompression.capability.supportLevel == kSDISupportLevelNone){
        config_ui->Transfer_data_after_comp_checkBox->hide();
    }else {
        config_ui->Transfer_data_after_comp_checkBox->show();
    }
    int item_num = 0;
    if(sv->device_data.SleepTime.capability.supportLevel == kSDISupportLevelNone){
        config_ui->Sleep_Timer_label->hide();
        config_ui->Sleep_Timer_spinBox->hide();
    }else {
        config_ui->Sleep_Timer_label->show();
        config_ui->Sleep_Timer_spinBox->show();
        item_num++;
    }
    if(sv->device_data.AutoPowerOffTime.capability.supportLevel == kSDISupportLevelNone){
        config_ui->AC_Power_colon_label->hide();
        config_ui->AC_Power_label->hide();
        config_ui->AC_Power_comboBox->hide();
        config_ui->Power_Off_Timer_groupBox->setFixedHeight(60);
    }else {
        config_ui->AC_Power_colon_label->show();
        config_ui->AC_Power_label->show();
        config_ui->AC_Power_comboBox->show();
        item_num++;
    }
    if(sv->device_data.AutoPowerOffTimeBattery.capability.supportLevel == kSDISupportLevelNone){
        config_ui->Battery_Power_colon_label->hide();
        config_ui->Battery_Power_label->hide();
        config_ui->Battery_Power_comboBox->hide();
        config_ui->Power_Off_Timer_groupBox->setFixedHeight(60);
    }else {
        config_ui->Battery_Power_colon_label->show();
        config_ui->Battery_Power_label->show();
        config_ui->Battery_Power_comboBox->show();
        item_num++;
    }
    if(item_num == 0){
        config_ui->tabWidget->setTabEnabled(2, false);
    }else {
        config_ui->tabWidget->setTabEnabled(2, false);
    }
}
bool ConfigurationDialog::CheckOverDurationCounter(CONFIGURE_COUNTER_VALUE ConfigureCounter, bool* AlertAlready)
{
    if(ConfigureCounter.RollerCounter.select < ConfigureCounter.DurationCounter.select){
        *AlertAlready = false;
    }
    if(!*AlertAlready){
        if(ConfigureCounter.RollerCounter.select >= ConfigureCounter.DurationCounter.select){
            *AlertAlready = true;
            return true;
        }
    }
    return false;
}
bool ConfigurationDialog::CheckNearDurationCounter(CONFIGURE_COUNTER_VALUE ConfigureCounter, bool* AlertAlready)
{
    if(ConfigureCounter.RollerCounter.select < ConfigureCounter.DurationCounter.select - NEAREND){
        *AlertAlready = false;
    }
    if(!*AlertAlready){
        if(ConfigureCounter.RollerCounter.select >= ConfigureCounter.DurationCounter.select - NEAREND){
            *AlertAlready = true;
            return true;
        }
    }
    return false;
}
