//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  custommessagebox.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "custommessagebox.h"
#include "translationstring.h"
#include <QCheckBox>
CustomMessageBox::CustomMessageBox(QWidget* parent)
{
    parent_dialog = parent;
}
bool CustomMessageBox::CreateMessageBox(QString message, QString button_message, IconType type, bool use_doNotShow)
{
    QMessageBox msgBox(parent_dialog);
    msgBox.setText(message + "\n");
    msgBox.setWindowTitle(TranslationString::GetString().translation_str.MENU_001);
    msgBox.setStandardButtons(QMessageBox::Yes);
    msgBox.setDefaultButton(QMessageBox::Yes);
    msgBox.setButtonText(QMessageBox::Yes, button_message);
    switch (type){
        case Question:
            msgBox.setIcon(QMessageBox::Question);
            break;
        case Information:
            msgBox.setIcon(QMessageBox::Information);
            break;
        case Warning:
            msgBox.setIcon(QMessageBox::Warning);
            break;
        case Critical:
            msgBox.setIcon(QMessageBox::Critical);
            break;
    }
    if(use_doNotShow){
        QCheckBox *doNotShowCheckBox = new QCheckBox(TranslationString::GetString().translation_str.LINUX_C_UI_002, parent_dialog);
        doNotShowCheckBox->setGeometry(10, 30, 0, 300);
        msgBox.setCheckBox(doNotShowCheckBox);
        msgBox.exec();
        if(doNotShowCheckBox->checkState() == Qt::Checked ){
            return true;
        }
    }else {
        msgBox.exec();
    }
    return false;
}
void CustomMessageBox::ErrorMessageBox(Supervisor* sv)
{
    QString error_message;
    IconType type = Warning;
    if(sv->m_LastError != kSDIErrorNone){
        if(sv->m_LastError == kSDIErrorPaperEmpty){
            error_message = TranslationString::GetString().translation_str.E_SCAN_101;
        }else if(sv->m_LastError == kSDIErrorPaperJam){
            this->PaperJamDialog(sv, TranslationString::GetString().translation_str.CONFIRMCONTINUE_008);
            return;
        }else if(sv->m_LastError == kSDIErrorPaperDoubleFeed){
            this->PaperJamDialog(sv, TranslationString::GetString().translation_str.CONFIRMCONTINUE_002);
            return;
        }else if(sv->m_LastError == kSDIErrorPaperProtect){
            this->PaperJamDialog(sv, TranslationString::GetString().translation_str.CONFIRMCONTINUE_011);
            return;
        }else if(sv->m_LastError == kSDIErrorDeviceInBusy){
            error_message = "kSDIErrorDeviceInBusy";
        }else if(sv->m_LastError == kSDIErrorCoverOpen){
            error_message = TranslationString::GetString().translation_str.E_SCAN_102;
        }else if(sv->m_LastError == kSDIErrorTrayClose){
            error_message = TranslationString::GetString().translation_str.E_SCAN_105;
        }else if(sv->m_LastError == kSDIErrorInvalidResponse){
            error_message = "";
        }else if(sv->m_LastError == kSDIErrorNoMemory){
            type = Critical;
            error_message = TranslationString::GetString().translation_str.E_ENV_001;
            sv->scanning_status = ERROR;
        }else if(sv->m_LastError == kSDITransferEventTypeCancel){
            return;
        }else if(sv->m_LastError == kSDIErrorDataSend){
            type = Critical;
            error_message = TranslationString::GetString().translation_str.E_COMM_103;
            sv->scanning_status = ERROR;
        }else if(sv->m_LastError == kSDIErrorDataReceive){
            type = Critical;
            error_message = TranslationString::GetString().translation_str.E_COMM_102;
            sv->scanning_status = ERROR;
        }else if(sv->m_LastError == kSDIErrorDeviceInUse){
            type = Critical;
            error_message = TranslationString::GetString().translation_str.E_COMM_001;
            sv->scanning_status = ERROR;
        }else if(sv->m_LastError == kSDIErrorWriteFile){
            error_message = TranslationString::GetString().translation_str.E_SAVE_001;
        }else if(sv->m_LastError == kSDIErrorUnknownError){
            type = Critical;
            error_message = TranslationString::GetString().translation_str.E_SCAN_001;
            sv->scanning_status = ERROR;
        }else {
            type = Critical;
            error_message = TranslationString::GetString().translation_str.E_SCAN_001;
            sv->scanning_status = ERROR;
        }
    }else {
        return;
    }
    CreateMessageBox(error_message, TranslationString::GetString().translation_str.VERSION_003, Warning);
}
void CustomMessageBox::PaperJamDialog(Supervisor* sv, QString message)
{
    QMessageBox msgBox(parent_dialog);
    msgBox.setText(message);
    msgBox.setWindowTitle(TranslationString::GetString().translation_str.MENU_001);
    msgBox.setStandardButtons(QMessageBox::Close | QMessageBox::Yes | QMessageBox::No);
    msgBox.setDefaultButton(QMessageBox::No);
    msgBox.setButtonText(QMessageBox::Close, TranslationString::GetString().translation_str.CONFIRMCONTINUE_005);
    msgBox.setButtonText(QMessageBox::Yes, TranslationString::GetString().translation_str.CONFIRMCONTINUE_004);
    msgBox.setButtonText(QMessageBox::No, TranslationString::GetString().translation_str.CONFIRMCONTINUE_003);
    msgBox.setIcon(QMessageBox::Warning);
    int res = msgBox.exec();
    if (res == QMessageBox::Close){
        sv->scanning_status = CANCEL;
    } else if (res == QMessageBox::Yes){
        sv->Get_Image();
        sv->scanning_status = SAVE;
    } else if (res == QMessageBox::No) {
        sv->scanning_status = CONTINUE;
    }
}
