//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  deviceconnectiondialog.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "mainwindow.h"
#include "ui_deviceconnectiondialog.h"
#include "PathUtils.h"
#include <QDebug>
#include <QProcess>
#include <pthread.h>
DeviceConnectionDialog::DeviceConnectionDialog(Supervisor* sv, SDIDeviceInfo select_Device, bool use_lastsettings, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DeviceConnectionDialog)
{
    ui->setupUi(this);
    this->setFixedSize(this->size());
    this->m_connect_info.sv = sv;
    this->m_connect_info.select_Device = select_Device;
    this->m_connect_info.use_lastsettings = use_lastsettings;
    this->m_connect_info.dialog = this;
    this->Initialize();
}
DeviceConnectionDialog::~DeviceConnectionDialog()
{
    delete ui;
}
void DeviceConnectionDialog::closeEvent(QCloseEvent* event){
    event->ignore();
}
bool DeviceConnectionDialog::GetConnectionResult(void)
{
    return this->m_connect_info.connection_result;
}
void DeviceConnectionDialog::Initialize(void)
{
    pthread_t thread;
    if(this->isHidden()){
        pthread_create(&thread, NULL, &DeviceConnectionDialog::ConnectToDevice, &this->m_connect_info);
        this->exec();
        pthread_join(thread, NULL);
    }
}
void DetectAlertGlassDirty(SDIScannerDriver* driver, SDIInterruptEventType event , void* userdata)
{
    UNREFERENCED_PARAMETER(driver);
    UNREFERENCED_PARAMETER(userdata);
    QDir tmpdir("/tmp");
    QStringList nameFilters;
    nameFilters << "epsonscan2alert*";
    QStringList dirlist = tmpdir.entryList(nameFilters, QDir::Dirs);
    int dir_num = dirlist.count();
    std::string dir_name = "epsonscan2alert";
    if(event == kSDIInterruptEventTypeCleaningRequired){
        std::string strDetectAlertPath = EPSON_INSTALL_PATH;
        ES_CMN_FUNCS::PATH::ES_CombinePath(strDetectAlertPath, strDetectAlertPath, "detectalert");
        QString exeapp(strDetectAlertPath.c_str());
        QString arg1("EPSON");
        QString arg2(TranslationString::GetString().translation_str.A_UI_006);
        QString arg3(TranslationString::GetString().translation_str.VERSION_003);
        QString executable = exeapp + " " + arg1 + " " + "\"" + arg2 + "\"" + " " + arg3 + " " + "A_UI_006" + " " + std::to_string( 1).c_str();
        dir_name = dir_name + "A_UI_006";
        if(!tmpdir.exists(dir_name.c_str())){
            if(!tmpdir.exists(dir_name.c_str())) tmpdir.mkdir(dir_name.c_str());
            if(!QProcess::startDetached(executable)){
                qCritical() << "Cannot start program";
            }
        }
    }else if(event == kSDIInterruptEventTypeGlassDirty){
        std::string strDetectAlertPath = EPSON_INSTALL_PATH;
        ES_CMN_FUNCS::PATH::ES_CombinePath(strDetectAlertPath, strDetectAlertPath, "detectalert");
        QString exeapp(strDetectAlertPath.c_str());
        QString arg1("EPSON");
        QString arg2(TranslationString::GetString().translation_str.A_UNIT_008);
        QString arg3(TranslationString::GetString().translation_str.VERSION_003);
        QString executable = exeapp + " " + arg1 + " " + "\"" + arg2 + "\"" + " " + arg3 + " " + "A_UNIT_008" + " " + std::to_string( 2).c_str();
        dir_name = dir_name + "A_UNIT_008";
        if(!tmpdir.exists(dir_name.c_str())){
            if(!tmpdir.exists(dir_name.c_str())) tmpdir.mkdir(dir_name.c_str());
            if(!QProcess::startDetached(executable)){
                qCritical() << "Cannot start program";
            }
        }
    }
}
void* DeviceConnectionDialog::ConnectToDevice(void* vpContext)
{
    CONNECTINFO* connect_info = (CONNECTINFO*)vpContext;
    DeviceConnectionDialog* dialog = (DeviceConnectionDialog*)connect_info->dialog;
    Supervisor* sv = connect_info->sv;
    SDIDeviceInfo select_Device = connect_info->select_Device;
    bool use_lastsettings = connect_info->use_lastsettings;
    sleep(1);
    if(sv->Connecting_Scanner(select_Device, DetectAlertGlassDirty) == kSDIErrorNone){
        Configuration& config = Configuration::GetConfiguration();
        if(config.config_para.last_used_settings){
            if(!use_lastsettings){
                DetectAlertGlassDirty(NULL, kSDIInterruptEventTypeCleaningRequired, NULL);
                dialog->ResetOnlyStandaloneVal(sv);
            }
        }else {
            dialog->ResetOnlyStandaloneVal(sv);
        }
        connect_info->connection_result = true;
    }else {
        connect_info->connection_result = false;
    }
END:
    dialog->reject();
    return NULL;
}
void DeviceConnectionDialog::ResetOnlyStandaloneVal(Supervisor* sv)
{
    sv->device_data.ScanArea.FixedSize = Auto_Detect;
    sv->device_data.Add_Pages = false;
    sv->device_data.BlankPageSkip = false;
    sv->device_data.Prefix = DEFAULT_PREFIX;
    sv->device_data.ScanArea.UserDefine_Width = 0;
    sv->device_data.ScanArea.UserDefine_Length = 0;
    sv->device_data.ImageFormatSettings.PDFImgFormatOption.AllPage = true;
    sv->device_data.ImageFormatSettings.PDFImgFormatOption.page = 1;
    sv->device_data.UserDefinePath = "";
    sv->device_data.Folder = 0;
    sv->device_data.ImageOption = ImageOptionNone;
    sv->device_data.ScanArea.PaperEndDetectionMemory = false;
    sv->device_data.DNShow_PlasticCard_dialog = false;
    sv->device_data.DNShow_LongPaperWarning_dialog = false;
    if(this->search_item_list(kSDIFunctionalUnitFlatbed, sv->device_data.FunctionalUnit.capability.list, sv->device_data.FunctionalUnit.capability.countOfList) &&
       this->search_item_list(kSDIFunctionalUnitDocumentFeeder, sv->device_data.FunctionalUnit.capability.list, sv->device_data.FunctionalUnit.capability.countOfList)){
        sv->device_data.FunctionalUnit_Auto = true;
    }else {
        sv->device_data.FunctionalUnit_Auto = false;
    }
    sv->device_data.OverDurationAlert = false;
    sv->device_data.NearDurationAlert = false;
}
bool DeviceConnectionDialog::search_item_list(int item, int32_t item_list[], int32_t list_size)
{
    int i = 0;
    while(i < list_size){
        if(item_list[i] == item){
            return true;
        }
        i++;
    }
    return false;
}
