//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  documentsource.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "mainwindow.h"
Document_Source::Document_Source(QWidget *parent) :
    QComboBox(parent)
{
    changed = false;
}
void Document_Source::initialize(INT_SETTING_VALUE document_source_para, bool functionalunit_auto){
    changed = true;
    if(functionalunit_auto && this->search_item_list(kSDIFunctionalUnitFlatbed, document_source_para.capability.list, document_source_para.capability.countOfList)){
        current_item = Auto;
    }else {
        current_item = (FunctionalUnit)document_source_para.select;
    }
}
void Document_Source::update_ui(INT_SETTING_VALUE document_source_para)
{
    if(document_source_para.capability.supportLevel == kSDISupportLevelAvailable){
        this->set_enabled(true);
    }else {
        this->set_enabled(false);
    }
    if(changed == true){
        this->clear();
        this->add_item(document_source_para.capability.list, document_source_para.capability.countOfList);
        this->select_item();
        changed = false;
    }
}
void Document_Source::set_enabled(bool enabled)
{
    this->setEnabled(enabled);
}
void Document_Source::add_item(int32_t item_list[], int32_t list_size){
    int i = 0;
    while(i < list_size){
        if(item_list[i] == kSDIFunctionalUnitFlatbed){
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_004, Flatbed);
        }else if(item_list[i] == kSDIFunctionalUnitDocumentFeeder){
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_005, ADF);
        }else if(item_list[i] == kSDIFunctionalUnitCarrierSheet){
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_006, Carrier_Sheet);
        }
        i++;
    }
    if(this->findData(Flatbed) != -1 && this->findData(ADF) != -1){
        this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_003, Auto);
    }
}
void Document_Source::select_item()
{
    this->setCurrentIndex(this->findData(current_item));
}
void Document_Source::item_event(int value, INT_SETTING_VALUE* document_source_para, bool* functionalunit_auto)
{
    QVariant select_item = this->itemData(value);
    *functionalunit_auto = false;
    if(select_item == Auto){
        *functionalunit_auto = true;
        current_item = Auto;
    }else if(select_item == Flatbed){
        current_item = Flatbed;
        document_source_para->select = Flatbed;
    }else if(select_item == ADF){
        current_item = ADF;
        document_source_para->select = ADF;
    }else if(select_item == Carrier_Sheet){
        current_item = Carrier_Sheet;
        document_source_para->select = Carrier_Sheet;
    }
    changed = true;
}
bool Document_Source::search_item_list(int item, int32_t item_list[], int32_t list_size)
{
    int i = 0;
    while(i < list_size){
        if(item_list[i] == item){
            return true;
        }
        i++;
    }
    return false;
}
