//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  prefixlineedit.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "mainwindow.h"
PrefixLineedit::PrefixLineedit(QWidget *parent) :
    QLineEdit(parent)
{
}
void PrefixLineedit::initialize(std::string default_prefix){
    changed = true;
    current_prefix = default_prefix.c_str();
    this->setEnabled(true);
    this->setText(current_prefix);
}
QString PrefixLineedit::set_prefix(QString prefix){
    QRegExp r1("^.*[/:.*?\"<>|].*");
    if(r1.exactMatch(prefix) || prefix.indexOf("\\") != -1){
        BanMessageBox();
        current_prefix = DEFAULT_PREFIX;
        this->setText(DEFAULT_PREFIX);
    }else if(prefix.isEmpty()){
        current_prefix = DEFAULT_PREFIX;
    }else {
        current_prefix = prefix;
    }
    return current_prefix;
}
void PrefixLineedit::get_prefix(std::string* prefix){
    *prefix = (std::string)current_prefix.toLocal8Bit();
}
void PrefixLineedit::BanMessageBox(void)
{
    QMessageBox msgBox(this);
    msgBox.setText(TranslationString::GetString().translation_str.E_UI_103);
    msgBox.setWindowTitle(TranslationString::GetString().translation_str.MENU_001);
    msgBox.setStandardButtons(QMessageBox::Yes);
    msgBox.setDefaultButton(QMessageBox::Yes);
    msgBox.setButtonText(QMessageBox::Yes, TranslationString::GetString().translation_str.VERSION_003);
    msgBox.setIcon(QMessageBox::Warning);
    msgBox.exec();
}
