//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  scanbutton.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include <stdio.h>
#include <iostream>
#include <boost/format.hpp>
#include <QDesktopServices>
#include "scanbutton.h"
#include "mainwindow.h"
#include "scanningdialog.h"
#include "addpagedialog.h"
#include "saveimagepathlist.h"
#include "PathUtils.h"
#include "savedestfolder.h"
ScanButton::ScanButton(QWidget *parent) :
    QPushButton(parent)
{
    m_messageBox = NULL;
}
void ScanButton::initialize(CustomMessageBox* msg_box)
{
    this->setEnabled(true);
    m_messageBox = msg_box;
}
void ScanButton::Button_pressed(Supervisor* sv, bool Add_Pages)
{
    ScanningDialog* scanning_dialog = NULL;
    sv->scanning_status = SAVE;
    do{
        scanning_dialog = new ScanningDialog(sv);
        if(scanning_dialog == NULL){
            sv->scanning_status = ERROR;
            sv->m_LastError = kSDIErrorUnknownError;
            m_messageBox->ErrorMessageBox(sv);
            break;
        }
        scanning_dialog->setAttribute(Qt::WA_DeleteOnClose);
        if(sv->m_LastError != kSDIErrorNone){
            m_messageBox->ErrorMessageBox(sv);
            break;
        }else {
            if(scanning_dialog->isHidden()){
                scanning_dialog->exec();
            }else {
                scanning_dialog->activateWindow();
            }
        }
        if(sv->outEventType == kSDITransferEventTypeCancel && !Add_Pages){
            m_messageBox->ErrorMessageBox(sv);
            sv->scanning_status = CANCEL;
        }else
        if(sv->m_LastError != kSDIErrorNone){
            m_messageBox->ErrorMessageBox(sv);
            if(sv->m_LastError == kSDIErrorPaperEmpty || sv->m_LastError == kSDIErrorCoverOpen){
                sv->scanning_status = CANCEL;
            }else
            if(!(sv->m_LastError == kSDIErrorPaperJam ||
                 sv->m_LastError == kSDIErrorPaperDoubleFeed ||
                 sv->m_LastError == kSDIErrorPaperProtect)){
                sv->scanning_status = ERROR;
            }
        }else {
            if(Add_Pages){
                do{
                    AddPageDialog* add_page_dialog = new AddPageDialog(&sv->scanning_status);
                    if(add_page_dialog == NULL){
                        sv->scanning_status = ERROR;
                        sv->m_LastError = kSDIErrorUnknownError;
                        m_messageBox->ErrorMessageBox(sv);
                        break;
                    }
                    if(add_page_dialog->isHidden()){
                        add_page_dialog->exec();
                    }else {
                        add_page_dialog->activateWindow();
                    }
                    if(sv->scanning_status == SAVE){
                        sv->outEventType = kSDITransferEventTypeComplete;
                    }
                    if(sv->device_data.FunctionalUnit.select == kSDIFunctionalUnitDocumentFeeder &&
                            sv->scanning_status == CONTINUE){
                        if(!sv->GetPaperLoadedStatus()){
                            sv->m_LastError = kSDIErrorPaperEmpty;
                            m_messageBox->ErrorMessageBox(sv);
                        }else {
                            sv->m_LastError = kSDIErrorNone;
                        }
                    }else {
                        sv->m_LastError = kSDIErrorNone;
                    }
                }while(sv->m_LastError == kSDIErrorPaperEmpty);
            }else {
                sv->scanning_status = SAVE;
            }
        }
        if(sv->scanning_status == SAVE){
            if(sv->Save_Image2()){
                Configuration& config = Configuration::GetConfiguration();
                if(config.config_para.open_img_folder){
                    SaveDestFolder* folder = new SaveDestFolder();
                    if(folder){
                        if(!folder->open_folder(sv)){
                            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SAVE_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
                        }
                        delete folder;
                        folder = NULL;
                    }
                }else {
                    const bool result = ES_CMN_FUNCS::PATH::ES_IsExistFile(sv->device_data.SelectPath);
                    if (!result) {
                        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SAVE_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
                    }
                }
            }else {
                if(sv->m_LastError == kSDIErrorWriteFile){
                    m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SAVE_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
                }else {
                    m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_SCAN_001, TranslationString::GetString().translation_str.VERSION_003, Warning);
                }
            }
        }
        if(sv->scanning_status == CANCEL){
            while(sv->outEventType != kSDITransferEventTypeComplete){
                sv->Scanning();
                sv->Dispose_Ptr();
            }
            SaveImagePathList::save_image_path_list.clear();
        }
    }while(sv->scanning_status == CONTINUE);
}
